var ZeedhiFilters = angular.module("ZeedhiFilters", []);

ZeedhiFilters.filter("i18n", function(ScreenService) {
	return function(word, words, lang, variables) {
		return ScreenService.i18n(word, variables);
	};
});

ZeedhiFilters.filter("columnFormat", function(maskEngine) {
	return function(input, column) {
		var result = input;
		if (column && column.mask) {
			var mappedMasks = maskEngine.getMappedMasks();
			var params = column.mask.params || {};
			var maskFormatMethod = mappedMasks[column.mask.type] ? mappedMasks[column.mask.type].format : null;

			if (maskFormatMethod) {
				result = maskFormatMethod.apply(maskEngine, [input, params]);
			}
		}
		return result;
	};
});
/**
	* Filters out all duplicate items from an array by checking the specified key
	* @param [key] {string} the name of the attribute of each object to compare for uniqueness
	if the key is empty, the entire object will be compared
	if the key === false then no filtering will be performed
	* @return {array}
*/
ZeedhiFilters.filter('unique', function() {

	return function(items, filterOn) {

		if (filterOn === false) {
			return items;
		}

		if ((filterOn || angular.isUndefined(filterOn)) && angular.isArray(items)) {
			var newItems = [];

			var extractValueToCompare = function(item) {
				if (angular.isObject(item) && angular.isString(filterOn)) {
					return item[filterOn];
				} else {
					return item;
				}
			};

			angular.forEach(items, function(item) {
				var isDuplicate = false;
				for (var i = 0; i < newItems.length; i++) {
					if (angular.equals(extractValueToCompare(newItems[i]), extractValueToCompare(item))) {
						isDuplicate = true;
						break;
					}
				}
				if (!isDuplicate) {
					newItems.push(item);
				}
			});
			items = newItems;
		}
		return items;
	};
});

ZeedhiFilters.filter('orderByHeader', ["$filter", function($filter) {
	return function(data, orderValue, reverse) {
		if (orderValue) {
			data = $filter('orderBy')(data, orderValue, reverse);
		}
		return data;
	};
}]);

ZeedhiFilters.filter('hasWidgetToExpand', function() {
	return function(widgets) {
		if (widgets) {
			var expand = widgets.filter(function(widget) {
				return widget.expand === true;
			});
			if (expand.length > 0) {
				return expand;
			} else {
				return widgets;
			}
		}
	};
});

ZeedhiFilters.filter('groupBy', function() {
	var results = {};
	return function(data, key) {
		if (!(data && key)) return;
		var result;
		if (!this.$id) {
			result = {};
		} else {
			var scopeId = this.$id;
			if (!results[scopeId]) {
				results[scopeId] = {};
				this.$on("$destroy", function() {
					delete results[scopeId];
				});
			}
			result = results[scopeId];
		}

		for (var groupKey in result)
			result[groupKey].splice(0, result[groupKey].length);

		for (var i = 0; i < data.length; i++) {
			if (!result[data[i][key]])
				result[data[i][key]] = [];
			result[data[i][key]].push(data[i]);
		}

		var keys = Object.keys(result);
		for (var k = 0; k < keys.length; k++) {
			if (result[keys[k]].length === 0)
				delete result[keys[k]];
		}
		return result;
	};
});
ZeedhiFilters.filter('startsWith', function() {
	return function(items, column, query) {
		if (items) {
			return items.filter(function(item) {
				return query === undefined || (item[column] && item[column].toLowerCase().indexOf(query.toLowerCase()) === 0);
			});
		} else {
			return items;
		}
	};
});

ZeedhiFilters.filter('isLike', function() {
	return function(items, column, query) {
		if (items) {
			return items.filter(function(item) {
				return query === undefined || (item[column] && item[column].toLowerCase().indexOf(query.toLowerCase()) !== -1);
			});
		} else {
			return items;
		}
	};
});

ZeedhiFilters.filter('range', function() {
	return function(input, start, end) {
		input = [];
		for (var i = start; i <= end; i++) {
			input.push(i);
		}
		return input;
	};
});

ZeedhiFilters.filter('contains', function() {
	return function(items, column, query) {
		if (items && query) {

			return items.filter(function(item) {
				var filter = query === undefined ? ".*" : query.toLowerCase().replace(" ", ".*");
				var regExp = new RegExp(".*" + filter + ".*", "g");
				var result = query === undefined || (item[column] && regExp.exec(item[column].toLowerCase()) !== null);
				return result;
			});
		} else {
			return items;
		}
	};
});

ZeedhiFilters.filter('paginate', function() {
	return function(input, start, end) {
		if (input)
			return input.slice(start, end);
		else
			return [];
	};
});

ZeedhiFilters.filter('reverse', function() {
	return function(items) {
		return items.slice().reverse();
	};
});

ZeedhiFilters.filter('markLastInlineEditable', function() {
	return function markLastInlineEditable(array) {
		var field;
		var fieldIsVisible, fieldIsEditable;
		for (var i = array.length - 1; i >= 0; i--) {
			field = array[i];
			fieldIsVisible = field.aclIsVisible && field.isVisible;
			fieldIsEditable = field.inlineEditable && field.inlineEditable !== 'checkbox';
			if (fieldIsVisible && fieldIsEditable) {
				field._isLastInlineEditable = true;
				break;
			}
		}
		return array;
	};
});

ZeedhiFilters.filter('orderBy', function($parse) {
	return function(array, sortPredicate, reverseOrder) {
		if (!(Util.isArrayLike(array))) return array;
		sortPredicate = angular.isArray(sortPredicate) ? sortPredicate : [sortPredicate];

		if (sortPredicate.length === 0) {
			sortPredicate = ['+'];
		}
		sortPredicate = map(sortPredicate, function(predicate) {
			var descending = false,
				get = predicate || angular.identity,
				orderType = '';
			if (angular.isString(predicate)) {
				if ((predicate.charAt(0) == '+' || predicate.charAt(0) == '-')) {
					descending = predicate.charAt(0) == '-';
					predicate = predicate.substring(1);
				}
				if (predicate.indexOf('.') !== -1) {
					var items = predicate.split('.');
					predicate = items[0];
					orderType = items[1];
				}
				if (predicate === '') {
					// Effectively no predicate was passed so we compare identity
					return reverseComparator(function(a, b) {
						return compare(a, b, orderType);
					}, descending);
				}
				get = $parse(predicate);
				if (get.constant) {
					var key = get();
					return reverseComparator(function(a, b) {
						return compare(a[key], b[key], orderType);
					}, descending);
				}
			}
			return reverseComparator(function(a, b) {
				return compare(get(a), get(b), orderType);
			}, descending);
		});
		return Util.mergeSort(array, reverseComparator(comparator, reverseOrder));

		function comparator(o1, o2) {
			for (var i = 0; i < sortPredicate.length; i++) {
				var comp = sortPredicate[i](o1, o2);
				if (comp !== 0) return comp;
			}
			return 0;
		}

		function reverseComparator(comp, descending) {
			return Util.toBoolean(descending) ? function(a, b) {
				return comp(b, a);
			} : comp;
		}

		function map(obj, iterator, context) {
			var results = [];
			angular.forEach(obj, function(value, index, list) {
				results.push(iterator.call(context, value, index, list));
			});
			return results;
		}

		function compare(v1, v2, orderType) {
			var t1 = typeof v1;
			var t2 = typeof v2;
			if (t1 == t2) {
				if (Util.isDate(v1) && Util.isDate(v2)) {
					v1 = v1.valueOf();
					v2 = v2.valueOf();
					if (isNaN(v1) && isNaN(v2)) {
						return 0;
					} else if (isNaN(v1)) {
						return -1;
					} else if (isNaN(v2)) {
						return 1;
					}
				}
				if (t1 == "string") {
					if(!isNaN(v1) && !isNaN(v2) && orderType !== 'string'){
						v1 = Number(v1);
						v2 = Number(v2);
					}
					else {
						v1 = Util.removeAccents(v1.toLowerCase());
						v2 = Util.removeAccents(v2.toLowerCase());
					}
				}
				if (v1 === v2) return 0;
				return v1 < v2 ? -1 : 1;
			} else {
				return t1 < t2 ? -1 : 1;
			}
		}
	};
});
